package gov.grants.apply.dao;

import gov.grants.apply.valueObject.Application;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Vector;



 /**
  * @author Feng Qian
  *
  * Application Data Access Object (DAO).
  * This class contains all database handling that is needed to
  * permanently store and retrieve Applicatin object instances.
  */


public class ApplicationDao {


    /**
     * load-method. This will load valueObject contents from database using
     * Primary-Key as identifier. Upper layer should use this so that valueObject
     * instance is created and only primary-key should be specified. Then call
     * this method to complete other persistent information. This method will
     * overwrite all other fields except primary-key and possible runtime variables.
     * If load can not find matching row, NotFoundException will be thrown.
     *
     * @param conn         This method requires working database connection.
     * @param valueObject  This parameter contains the class instance to be loaded.
     *                     Primary-key field must be set for this to work properly.
     */
    public void load(Connection conn, Application valueObject) throws NotFoundException, SQLException {

          String sql = "SELECT * FROM application WHERE (OID = ? ) "; 
          PreparedStatement stmt = null;

          try {
               stmt = conn.prepareStatement(sql);
               stmt.setInt(1, valueObject.getOID()); 

               singleQuery(conn, stmt, valueObject);

          } finally {
              if (stmt != null)
                  stmt.close();
          }
    }


    /**
     * LoadAll-method. This will read all contents from database table and
     * build an Vector containing valueObjects. Please note, that this method
     * will consume huge amounts of resources if table has lot's of rows. 
     * This should only be used when target tables have only small amounts
     * of data.
     *
     * @param conn         This method requires working database connection.
     */
    public List loadAll(Connection conn) throws SQLException {

          String sql = "SELECT * FROM application ORDER BY OID ASC ";
          List searchResults = listQuery(conn, conn.prepareStatement(sql));

          return searchResults;
    }



    /**
     * create-method. This will create new row in database according to supplied
     * valueObject contents. Make sure that values for all NOT NULL columns are
     * correctly specified. Also, if this table does not use automatic surrogate-keys
     * the primary-key must be specified. After INSERT command this method will 
     * read the generated primary-key back to valueObject if automatic surrogate-keys
     * were used. 
     *
     * @param conn         This method requires working database connection.
     * @param valueObject  This parameter contains the class instance to be created.
     *                     If automatic surrogate-keys are not used the Primary-key 
     *                     field must be set for this to work properly.
     */
    public synchronized void create(Connection conn, Application valueObject) throws SQLException {

          String sql = "";
          PreparedStatement stmt = null;
          ResultSet result = null;

          try {
               sql = "INSERT INTO application ( TRACKING_ID, OPPORTUNITY_ID, CFDA_NUMBER, "
               + "COMPETITION_ID, RECEIVED_DATE, STATUS, AGENCY_TRACKING_ID, LAST_MODIFY_DATE, ERROR_LOG, SUBMISSION_TITLE ) " +
			   " VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?) ";
               stmt = conn.prepareStatement(sql);

               stmt.setString(1, valueObject.getTRACKING_ID()); 
               stmt.setString(2, valueObject.getOPPORTUNITY_ID()); 
               stmt.setString(3, valueObject.getCFDA_NUMBER()); 
               stmt.setString(4, valueObject.getCOMPETITION_ID()); 
               stmt.setLong(5, valueObject.getRECEIVED_DATE()); 
               stmt.setString(6, valueObject.getSTATUS()); 
               stmt.setString(7, valueObject.getAGENCY_TRACKING_ID());
               stmt.setLong(8, valueObject.getLAST_MODIFY_DATE() );
               stmt.setString(9, valueObject.getERROR_LOG() );
               stmt.setString(10, valueObject.getSUBMISSION_TITLE() );

               int rowcount = databaseUpdate(conn, stmt);
               if (rowcount != 1) {
                    //System.out.println("PrimaryKey Error when updating DB!");
                    throw new SQLException("PrimaryKey Error when updating DB!");
               }

          } finally {
              if (stmt != null)
                  stmt.close();
          }


          /**
          * The following query will read the automatically generated primary key
          * back to valueObject. This must be done to make things consistent for 
          * upper layer processing logic.
          */
          sql = "SELECT last_insert_id()";

          try {
              stmt = conn.prepareStatement(sql);
              result = stmt.executeQuery();

              if (result.next()) {

                   valueObject.setOID((int)result.getLong(1));

              } else {
                   //System.out.println("Unable to find primary-key for created object!");
                   throw new SQLException("Unable to find primary-key for created object!");
              }
          } finally {
              if (result != null)
                  result.close();
              if (stmt != null)
                  stmt.close();
          }

    }


    /**
     * save-method. This method will save the current state of valueObject to database.
     * Save can not be used to create new instances in database, so upper layer must
     * make sure that the primary-key is correctly specified. Primary-key will indicate
     * which instance is going to be updated in database. If save can not find matching 
     * row, NotFoundException will be thrown.
     *
     * @param conn         This method requires working database connection.
     * @param valueObject  This parameter contains the class instance to be saved.
     *                     Primary-key field must be set for this to work properly.
     */
    public void save(Connection conn, Application valueObject) 
          throws NotFoundException, SQLException {

          String sql = "UPDATE application SET TRACKING_ID = ?, OPPORTUNITY_ID = ?, CFDA_NUMBER = ?, "
               + "COMPETITION_ID = ?, RECEIVED_DATE = ?, STATUS = ?, ERROR_LOG = ?, SUBMISSION_TITLE = ? WHERE (OID = ? ) ";
          PreparedStatement stmt = null;

          try {
              stmt = conn.prepareStatement(sql);
              stmt.setString(1, valueObject.getTRACKING_ID()); 
              stmt.setString(2, valueObject.getOPPORTUNITY_ID()); 
              stmt.setString(3, valueObject.getCFDA_NUMBER()); 
              stmt.setString(4, valueObject.getCOMPETITION_ID()); 
              stmt.setLong(5, valueObject.getRECEIVED_DATE()); 
              stmt.setString(6, valueObject.getSTATUS()); 
              stmt.setString(7, valueObject.getERROR_LOG()); 
              stmt.setString(8, valueObject.getSUBMISSION_TITLE()); 
              
              stmt.setInt(9, valueObject.getOID()); 

              int rowcount = databaseUpdate(conn, stmt);
              if (rowcount == 0) {
                   //System.out.println("Object could not be saved! (PrimaryKey not found)");
                   throw new NotFoundException("Object could not be saved! (PrimaryKey not found)");
              }
              if (rowcount > 1) {
                   //System.out.println("PrimaryKey Error when updating DB! (Many objects were affected!)");
                   throw new SQLException("PrimaryKey Error when updating DB! (Many objects were affected!)");
              }
          } finally {
              if (stmt != null)
                  stmt.close();
          }
    }


    /**
     * delete-method. This method will remove the information from database as identified by
     * by primary-key in supplied valueObject. Once valueObject has been deleted it can not 
     * be restored by calling save. Restoring can only be done using create method but if 
     * database is using automatic surrogate-keys, the resulting object will have different 
     * primary-key than what it was in the deleted object. If delete can not find matching row,
     * NotFoundException will be thrown.
     *
     * @param conn         This method requires working database connection.
     * @param valueObject  This parameter contains the class instance to be deleted.
     *                     Primary-key field must be set for this to work properly.
     */
    public void delete(Connection conn, Application valueObject) 
          throws NotFoundException, SQLException {

          String sql = "DELETE FROM application WHERE (OID = ? ) ";
          PreparedStatement stmt = null;

          try {
              stmt = conn.prepareStatement(sql);
              stmt.setInt(1, valueObject.getOID()); 

              int rowcount = databaseUpdate(conn, stmt);
              if (rowcount == 0) {
                   //System.out.println("Object could not be deleted (PrimaryKey not found)");
                   throw new NotFoundException("Object could not be deleted! (PrimaryKey not found)");
              }
              if (rowcount > 1) {
                   //System.out.println("PrimaryKey Error when updating DB! (Many objects were deleted!)");
                   throw new SQLException("PrimaryKey Error when updating DB! (Many objects were deleted!)");
              }
          } finally {
              if (stmt != null)
                  stmt.close();
          }
    }


    /**
     * deleteAll-method. This method will remove all information from the table that matches
     * this Dao and ValueObject couple. This should be the most efficient way to clear table.
     * Once deleteAll has been called, no valueObject that has been created before can be 
     * restored by calling save. Restoring can only be done using create method but if database 
     * is using automatic surrogate-keys, the resulting object will have different primary-key 
     * than what it was in the deleted object. (Note, the implementation of this method should
     * be different with different DB backends.)
     *
     * @param conn         This method requires working database connection.
     */
    public void deleteAll(Connection conn) throws SQLException {

          String sql = "DELETE FROM application";
          PreparedStatement stmt = null;

          try {
              stmt = conn.prepareStatement(sql);
              int rowcount = databaseUpdate(conn, stmt);
          } finally {
              if (stmt != null)
                  stmt.close();
          }
    }


    /**
     * coutAll-method. This method will return the number of all rows from table that matches
     * this Dao. The implementation will simply execute "select count(primarykey) from table".
     * If table is empty, the return value is 0. This method should be used before calling
     * loadAll, to make sure table has not too many rows.
     *
     * @param conn         This method requires working database connection.
     */
    public int countAll(Connection conn) throws SQLException {

          String sql = "SELECT count(*) FROM application";
          PreparedStatement stmt = null;
          ResultSet result = null;
          int allRows = 0;

          try {
              stmt = conn.prepareStatement(sql);
              result = stmt.executeQuery();

              if (result.next())
                  allRows = result.getInt(1);
          } finally {
              if (result != null)
                  result.close();
              if (stmt != null)
                  stmt.close();
          }
          return allRows;
    }


    /** 
     * searchMatching-Method. This method provides searching capability to 
     * get matching valueObjects from database. It works by searching all 
     * objects that match permanent instance variables of given object.
     * Upper layer should use this by setting some parameters in valueObject
     * and then  call searchMatching. The result will be 0-N objects in vector, 
     * all matching those criteria you specified. Those instance-variables that
     * have NULL values are excluded in search-criteria.
     *
     * @param conn         This method requires working database connection.
     * @param valueObject  This parameter contains the class instance where search will be based.
     *                     Primary-key field should not be set.
     */
    public List searchMatching(Connection conn, Application valueObject) throws SQLException {

          List searchResults;

          boolean first = true;
          StringBuffer sql = new StringBuffer("SELECT * FROM application WHERE 1=1 ");

          if (valueObject.getOID() != 0) {
              if (first) { first = false; }
              sql.append("AND OID = ").append(valueObject.getOID()).append(" ");
          }

          if ((valueObject.getTRACKING_ID() != null) && (valueObject.getTRACKING_ID().length() > 0)) {
              if (first) { first = false; }
              sql.append("AND TRACKING_ID = '").append(valueObject.getTRACKING_ID()).append("' ");
          }

          if ((valueObject.getOPPORTUNITY_ID() != null) && (valueObject.getOPPORTUNITY_ID().length() > 0)) {
              if (first) { first = false; }
              sql.append("AND OPPORTUNITY_ID = '").append(valueObject.getOPPORTUNITY_ID()).append("' ");
          }

          if ((valueObject.getCFDA_NUMBER() != null) && (valueObject.getCFDA_NUMBER().length() > 0)) {
              if (first) { first = false; }
              sql.append("AND CFDA_NUMBER = '").append(valueObject.getCFDA_NUMBER()).append("' ");
          }

          if ((valueObject.getCOMPETITION_ID() != null)
				&& (valueObject.getCOMPETITION_ID().length() > 0)){
              if (first) { first = false; }
              sql.append("AND COMPETITION_ID = '").append(valueObject.getCOMPETITION_ID()).append("' ");
          }

          if (valueObject.getRECEIVED_DATE() != 0) {
              if (first) { first = false; }
              sql.append("AND RECEIVED_DATE = '").append(valueObject.getRECEIVED_DATE()).append("' ");
          }
          if (valueObject.getSTATUS() != null) {
          if (first) { first = false; }
          sql.append("AND STATUS LIKE '").append(valueObject.getSTATUS()).append("%' ");
          }
          if (valueObject.getERROR_LOG() != null) {
            if (first) { first = false; }
            sql.append("AND ERROR_LOG = '").append(valueObject.getERROR_LOG()).append("' ");
        }
          
          if ((valueObject.getSUBMISSION_TITLE() != null) && (valueObject.getSUBMISSION_TITLE().length() > 0)){
            if (first) { first = false; }
            sql.append("AND SUBMISSION_TITLE = '").append(valueObject.getSUBMISSION_TITLE()).append("' ");
        }

          sql.append("ORDER BY OID ASC ");

          // Prevent accidential full table results.
          // Use loadAll if all rows must be returned.
          if (first)
               searchResults = new Vector();
          else
               searchResults = listQuery(conn, conn.prepareStatement(sql.toString()));

          return searchResults;
    }


    /** 
     * getDaogenVersion will return information about
     * generator which created these sources.
     */
    public String getDaogenVersion() {
        return "DaoGen version 2.2.2";
    }


    /**
     * databaseUpdate-method. This method is a helper method for internal use. It will execute
     * all database handling that will change the information in tables. SELECT queries will
     * not be executed here however. The return value indicates how many rows were affected.
     * This method will also make sure that if cache is used, it will reset when data changes.
     *
     * @param conn         This method requires working database connection.
     * @param stmt         This parameter contains the SQL statement to be excuted.
     */
    protected int databaseUpdate(Connection conn, PreparedStatement stmt) throws SQLException {

          int result = stmt.executeUpdate();

          return result;
    }



    /**
     * databaseQuery-method. This method is a helper method for internal use. It will execute
     * all database queries that will return only one row. The resultset will be converted
     * to valueObject. If no rows were found, NotFoundException will be thrown.
     *
     * @param conn         This method requires working database connection.
     * @param stmt         This parameter contains the SQL statement to be excuted.
     * @param valueObject  Class-instance where resulting data will be stored.
     */
    protected void singleQuery(Connection conn, PreparedStatement stmt, Application valueObject) 
          throws NotFoundException, SQLException {

          ResultSet result = null;

          try {
              result = stmt.executeQuery();

              if (result.next()) {

                   valueObject.setOID(result.getInt("OID")); 
                   valueObject.setTRACKING_ID(result.getString("TRACKING_ID")); 
                   valueObject.setOPPORTUNITY_ID(result.getString("OPPORTUNITY_ID")); 
                   valueObject.setCFDA_NUMBER(result.getString("CFDA_NUMBER")); 
                   valueObject.setCOMPETITION_ID(result.getString("COMPETITION_ID")); 
                   valueObject.setRECEIVED_DATE(result.getLong("RECEIVED_DATE")); 
                   valueObject.setSTATUS(result.getString("STATUS")); 
                   valueObject.setAGENCY_TRACKING_ID( result.getString("AGENCY_TRACKING_ID"));
                   valueObject.setLAST_MODIFY_DATE(result.getLong( "LAST_MODIFY_DATE"));
                   valueObject.setERROR_LOG(result.getString( "ERROR_LOG"));
                   valueObject.setSUBMISSION_TITLE(result.getString( "SUBMISSION_TITLE"));

              } else {
                    //System.out.println("Application Object Not Found!");
                    throw new NotFoundException("Application Object Not Found!");
              }
          } finally {
              if (result != null)
                  result.close();
              if (stmt != null)
                  stmt.close();
          }
    }


    /**
     * databaseQuery-method. This method is a helper method for internal use. It will execute
     * all database queries that will return multiple rows. The resultset will be converted
     * to the List of valueObjects. If no rows were found, an empty List will be returned.
     *
     * @param conn         This method requires working database connection.
     * @param stmt         This parameter contains the SQL statement to be excuted.
     */
    protected List listQuery(Connection conn, PreparedStatement stmt) throws SQLException {

          Vector searchResults = new Vector();
          ResultSet result = null;

          try {
              result = stmt.executeQuery();

              while (result.next()) {
                   Application temp = new Application();

                   temp.setOID(result.getInt("OID")); 
                   temp.setTRACKING_ID(result.getString("TRACKING_ID")); 
                   temp.setOPPORTUNITY_ID(result.getString("OPPORTUNITY_ID")); 
                   temp.setCFDA_NUMBER(result.getString("CFDA_NUMBER")); 
                   temp.setCOMPETITION_ID(result.getString("COMPETITION_ID")); 
                   temp.setRECEIVED_DATE(result.getLong("RECEIVED_DATE")); 
                   temp.setSTATUS(result.getString("STATUS")); 
                   temp.setAGENCY_TRACKING_ID( result.getString( "AGENCY_TRACKING_ID"));
                   temp.setLAST_MODIFY_DATE( result.getLong("LAST_MODIFY_DATE"));
                   temp.setERROR_LOG( result.getString("ERROR_LOG"));
                   temp.setSUBMISSION_TITLE( result.getString("SUBMISSION_TITLE"));
                   
                   searchResults.add(temp);
              }

          } finally {
              if (result != null)
                  result.close();
              if (stmt != null)
                  stmt.close();
          }

          return (List)searchResults;
    }


}



